;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "agent-shell" "20260829.1116"
  "Native agentic integrations for Claude Code, Gemini CLI, etc."
  '((emacs       "29.1")
    (shell-maker "0.97.2")
    (acp         "0.13.1"))
  :url "https://github.com/xenodium/agent-shell"
  :commit "f28ca5f5e7eb718de0884412f672e4db5e01234a"
  :revdesc "f28ca5f5e7eb")
